#include <OneWire.h>
#include <DallasTemperature.h>
#include <CayenneLPP.h>
#include <string>
#include <MKRWAN_v2.h>
#include <ArduinoLowPower.h>

// Nastavení barev diod
const int NONE = 0;
const int GREEN = 1;
const int ORANGE = 2;
const int RED = 3;

// Definice pinů pro 
const int teplota_PIN = 0;
const int baterie_PIN = A1;
const int tlacitko_PIN = A2;
const int kontakt_PIN = 1;
const int kontakt_zem_PIN = A3;

const int LED_1_PIN = 12;
const int LED_2_PIN = 13;
const int LED_3_PIN = 14;

int stav_zpravy = 0;
// periodické odesílání zpráv po X sekundách
int keep_alive_timeout = 1800;

// Parametry pro ABP ověření
const char *devAddr = "";
const char *nwkSKey = "";
const char *appSKey = "";

// Parametry pro OTAA ověření
//const char *appEui = "";
//const char *appKey = "";

LoRaModem modem;
CayenneLPP lpp(51);
OneWire oneWire(teplota_PIN);
DallasTemperature sensors(&oneWire);

void setup() {
    // Nastavení referenčního napětí
    analogReference(AR_INTERNAL1V0);
    // Inicializace vstupních/výstupních PINů
    pinMode(tlacitko_PIN, INPUT);
    pinMode(kontakt_PIN, INPUT_PULLUP);
    pinMode(5, INPUT_PULLUP); 
    pinMode(LED_BUILTIN, OUTPUT);
    pinMode(kontakt_zem_PIN, OUTPUT);
    pinMode(LED_1_PIN, OUTPUT);
    pinMode(LED_2_PIN, OUTPUT);
    pinMode(LED_3_PIN, OUTPUT);
    // Otevření sériového portu a nastavení data rate na 9600 bps
    Serial.begin(9600);
    // Rozsvícení oranžové diody
    status_led(ORANGE);
    Serial.println("Čekám na stisktnutí tlačítka.");
    // Cekani na stisknutí tlačítka
    while (digitalRead(tlacitko_PIN)){}
    // Pokud se nepovede zapnout radio, rozsvítí se červená dioda a po řs se zařízení restaruje
    if (!modem.begin(EU868)) {
        Serial.println("Nelze spustit radio modul.");
        status_led(RED);
        delay(5000);
        NVIC_SystemReset();
    };
    // Pokud je radio spuštěno, rozsvítí se zelená dioda
    Serial.println("Radio modul spuštěn.");
    status_led(GREEN);
    int connected;
    connected = modem.joinABP(devAddr, nwkSKey, appSKey);
    //connected = modem.joinOTAA(appEui, appKey);
    if (!connected) {
        Serial.println("Nelze se připojit k síti.");
        status_led(RED);
        delay(5000);
        NVIC_SystemReset();
    }
    Serial.println("Připojeno.");
    status_led(NONE);
    LowPower.attachInterruptWakeup(digitalPinToInterrupt(kontakt_PIN), dummy, CHANGE);
    LowPower.attachInterruptWakeup(keep_alive_timeout*1000, dummy, CHANGE);
}

void loop() {
    digitalWrite(LED_BUILTIN, HIGH);
    zprava();
    digitalWrite(LED_BUILTIN, LOW);
    delay(1000);
    LowPower.deepSleep(keep_alive_timeout*1000);
}

void zprava() {
  /* Metoda pro odeslání zprávy s daty.
   * Nejprve se resetuje instance CayenneLPP.
   * Pote se vyčtou hodnoty vstupů.
   * Přidají se do instance CayenneLPP se 
   * zvoleným kanálem a typem hodnoty.
   * Aktivuje se modem, do nového paketu se zapíše 
   * instance s hodnotami o dané velikosti.
   * Paket se odešle (false = Unconfirmed Data Up)
   */
    lpp.reset();
    int kontakt_hodnota = kontakt();
    float teplota_hodnota = teplota();
    float baterie_hodnota = baterie();
    delay(500);
    lpp.addDigitalOutput(1, kontakt_hodnota);
    lpp.addTemperature(2, teplota_hodnota);
    lpp.addAnalogInput(3, baterie_hodnota);
    modem.setPort(3);
    modem.beginPacket();
    modem.write(lpp.getBuffer(), lpp.getSize());
    modem.endPacket(false);
}

float teplota() {
    /* Vyčtení hodnoty z teplotního čidla  
     *  Nejprve se aktivuje pin, ke kterému je čidlo připojeno,
     *  poté se vyčte hodnota z prvního senzoru na 1-wire sběrnici.
     *  Jako první na sběrnici je externí teplotní čidlo,
     *  pokud není připojeno, je první hodnotou interní čidlo.
     */
    pinMode(teplota_PIN, INPUT);
    sensors.begin();
    sensors.requestTemperatures();
    float teplota_value = sensors.getTempCByIndex(0);
    pinMode(teplota_PIN, OUTPUT);
    return teplota_value;
}

int kontakt() {
    // reverzní hodnota na kontakt_PIN pro stav monitorovaného zařízení
    int kontakt_raw = digitalRead(kontakt_PIN);
    return (kontakt_raw) ? 0 : 1;
}

float baterie() {
    // Vyčtení hodnoty na "baterie_PIN" a vynásobení koeficientem pro získání napětí ve V
    return analogRead(baterie_PIN) * (4.7 / 1023.0);
}

void status_led(int status){
    // Vypnutí všech diod a poté zapnutí dle předaného parametru
    digitalWrite(LED_1_PIN, LOW);
    digitalWrite(LED_2_PIN, LOW);
    digitalWrite(LED_3_PIN, LOW);
    switch (status) {
        case 1:
            digitalWrite(LED_1_PIN, HIGH);
            break;
        case 2:
            digitalWrite(LED_2_PIN, HIGH);
            break;
        case 3:
            digitalWrite(LED_3_PIN, HIGH);
            break;
    }
}

void dummy() {
}
